package com.swagger.rest.controllers

import com.swagger.rest.models.Platform
import com.swagger.rest.models.PlatformInput
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.data.domain.Page
import org.springframework.data.domain.PageRequest
import org.springframework.data.domain.Pageable
import org.springframework.data.domain.Sort
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api/v1")
class PlatformController(
    private val platformRepository: PlatformRepository,
    private val projectRepository: ProjectRepository,
    private val memberRepository: MemberRepository,
    private val userRepository: UserRepository
) {
    private fun getSortDirection(direction: String): Sort.Direction {
        if (direction == "asc") {
            return Sort.Direction.ASC
        } else if (direction == "desc") {
            return Sort.Direction.DESC
        }
        return Sort.Direction.ASC
    }

    @GetMapping("/platforms")
    fun getPlatformByProjectId(
        @RequestParam(required = false) project: String?,
        @RequestParam(defaultValue = 0.toString()) page: Int,
        @RequestParam(defaultValue = 3.toString()) limit: Int,
        @RequestParam(defaultValue = "id, desc") sort: Array<String>
    ): Any {
        return try {
            val orders: MutableList<Sort.Order> = ArrayList()
            if (sort[0].contains(",")) {
                for (sortOrder in sort) {
                    val sort_ = sortOrder.split(",".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                    orders.add(Sort.Order(getSortDirection(sort_[1]), sort_[0]))
                }
            } else {
                orders.add(Sort.Order(Sort.Direction.DESC, sort[0]))
            }
            val pagingSort: Pageable = PageRequest.of(page, limit, Sort.by(orders as List<Sort.Order>))
            val platforms: Page<Platform> = if (project == null) {
                platformRepository.findAll(pagingSort)
            } else {
                platformRepository.findByProject(project, pagingSort)
            }
            val ret: List<Platform?> = platforms.content
            val response: MutableMap<String, Any> = HashMap()
            response["currentPage"] = platforms.number
            response["totalRecord"] = platforms.totalElements
            response["totalPage"] = platforms.totalPages
            response["results"] = ret
            if (ret.isNotEmpty()) {
                ResponseEntity(response, HttpStatus.OK)
            } else {
                arrayOf<String>()
            }
        } catch (e: Exception) {
            ResponseEntity(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @GetMapping("/platforms/{id}")
    fun getPlatformById(@PathVariable("id") id: Long): ResponseEntity<Platform> {
        val platformData: Optional<Platform?> = platformRepository.findById(id)
        return if (platformData.isPresent) {
            ResponseEntity<Platform>(platformData.get(), HttpStatus.OK)
        } else {
            ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
        }
    }

    @PostMapping("/platforms")
    fun addPlatformByProjectId(
        @RequestBody platform: PlatformInput
    ): ResponseEntity<Platform> {
        val proj = (projectRepository.countByName(platform.project_name))
        return if (proj > 0 && platform.project_name!!.isNotBlank()) {
            val foundProject = projectRepository.findByName(platform.project_name)
            val found = platformRepository.findByName(platform.name, foundProject.id.toString()).size
            val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            val validOwner = projectRepository.validOwner(foundProject.id.toString(), userId.id.toString())
            val validAdmin = memberRepository.validRole(foundProject.id.toString(), userId.id.toString(), "2")
            if (validOwner > 0 || validAdmin > 0) {
                if (platform.name!!.isNotBlank()) {
                    if (platform.name!!.length > 100) {//too long
                        ResponseEntity<Platform>(HttpStatus.PAYLOAD_TOO_LARGE)
                    } else if (found > 0) {//duplicate
                        ResponseEntity<Platform>(HttpStatus.CONFLICT)
                    } else {
                        val newPlatform = Platform()
                        newPlatform.name = platform.name!!.trim()
                        newPlatform.project_id = foundProject
                        ResponseEntity<Platform>(platformRepository.save(newPlatform), HttpStatus.CREATED)
                    }
                } else {//invalid name
                    ResponseEntity<Platform>(HttpStatus.BAD_REQUEST)
                }
            } else {
                ResponseEntity<Platform>(HttpStatus.FORBIDDEN)
            }
        } else {
            ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
        }
    }

    @PutMapping("/platforms/{id}")
    fun updatePlatformById(@PathVariable("id") id: Long, @RequestBody input: PlatformInput): Any {
        val targetProj = projectRepository.countByName(input.project_name)
        return if (targetProj > 0) {
            val targetProject = projectRepository.findByName(input.project_name)//target project
            val platformExist = platformRepository.findById(id)//exist data
            val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            val validOwner = projectRepository.validOwner(targetProject.id.toString(), userId.id.toString())
            val validAdmin = memberRepository.validRole(targetProject.id.toString(), userId.id.toString(), "2")
            if (validOwner > 0 || validAdmin > 0) {
                if (input.name!!.isNotBlank()) {
                    if (input.name!!.length > 100) {//too long
                        ResponseEntity<Platform>(HttpStatus.PAYLOAD_TOO_LARGE)
                    } else if (targetProject.name.isEmpty() || platformExist.isEmpty) {//target project not found
                        ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
                    } else {
                        val check = platformRepository.findByName(input.name, targetProject.id.toString())
                        if (check.isEmpty() || (input.name == platformExist.get().name && targetProject.id.toString() == platformExist.get().project_id!!.id.toString())) {//tidak ada yg sama
                            val savePlatform = platformExist.get()
                            savePlatform.project_id = targetProject
                            savePlatform.name = input.name!!.trim()
                            ResponseEntity<Platform>(platformRepository.save(savePlatform), HttpStatus.OK)
                        } else {
                            ResponseEntity<Platform>(HttpStatus.CONFLICT)
                        }
                    }
                } else {
                    ResponseEntity<Platform>(HttpStatus.BAD_REQUEST)
                }
            } else {//invalid name
                ResponseEntity<Platform>(HttpStatus.FORBIDDEN)
            }
        } else {
            ResponseEntity<Platform>(HttpStatus.NOT_FOUND)
        }
    }

    @DeleteMapping("/platforms/{id}")
    fun deletePlatformById(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val count = platformRepository.findById(id).isPresent
        return try {
            if (count) {
                val find = platformRepository.findById(id)
                val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
                val validOwner =
                    projectRepository.validOwner(find.get().project_id!!.id.toString(), userId.id.toString())
                val validAdmin =
                    memberRepository.validRole(find.get().project_id!!.id.toString(), userId.id.toString(), "2")
                if (validOwner > 0 || validAdmin > 0) {
                    if (find.isPresent) {
                        platformRepository.deleteById(id)
                        ResponseEntity(HttpStatus.OK)
                    } else {
                        ResponseEntity(HttpStatus.NOT_FOUND)
                    }
                } else {
                    ResponseEntity(HttpStatus.FORBIDDEN)
                }
            } else {
                ResponseEntity(HttpStatus.NOT_FOUND)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }
}
