package com.swagger.rest.controllers

import com.swagger.rest.models.Project
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.data.domain.PageRequest
import org.springframework.data.domain.Pageable
import org.springframework.data.domain.Sort
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api/v1")
class ProjectController(
    private val projectRepository: ProjectRepository,
    private val platformRepository: PlatformRepository,
    private val userRepository: UserRepository,
    private val memberRepository: MemberRepository
) {
    private fun getSortDirection(direction: String): Sort.Direction {
        if (direction == "asc") {
            return Sort.Direction.ASC
        } else if (direction == "desc") {
            return Sort.Direction.DESC
        }
        return Sort.Direction.ASC
    }

    @GetMapping("/projects")
    fun getProject(
        @RequestParam(required = false) name: String?,
        @RequestParam(defaultValue = 0.toString()) page: Int,
        @RequestParam(defaultValue = 3.toString()) limit: Int,
        @RequestParam(defaultValue = "id, desc") sort: Array<String>
    ): Any {
        return try {
            val orders: MutableList<Sort.Order> = ArrayList()
            if (sort[0].contains(",")) {
                for (sortOrder in sort) {
                    val sort_ = sortOrder.split(",".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
                    orders.add(Sort.Order(getSortDirection(sort_[1]), sort_[0]))
                }
            } else {
//                orders.add(Sort.Order(getSortDirection(sort[1]), sort[0]))
                orders.add(Sort.Order(Sort.Direction.DESC, sort[0]))
            }
            val pagingSort: Pageable = PageRequest.of(page, limit, Sort.by(orders as List<Sort.Order>))
            val projects =
                if (name == null) projectRepository.findAll(pagingSort) else projectRepository.findByNameContaining(
                    name, pagingSort
                )
            val ret = projects.content
            val response: MutableMap<String, Any> = HashMap()
            response["currentPage"] = projects.number
            response["totalRecord"] = projects.totalElements
            response["totalPage"] = projects.totalPages
            response["results"] = ret
            if (ret.isEmpty()) {
                arrayOf<String>()
            } else {
                ResponseEntity(response, HttpStatus.OK)
            }
        } catch (e: Exception) {
            ResponseEntity(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @GetMapping("/projects/{id}")
    fun getProjectById(@PathVariable("id") id: Long): ResponseEntity<Project> {
        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
        val validOwner = projectRepository.validOwner(id.toString(), userId.id.toString())
        val validMember = memberRepository.validMember(id.toString(), userId.id.toString())
        val projectData: Optional<Project?> = projectRepository.findById(id)
        return if (validOwner > 0 || validMember > 0) {
            if (projectData.isPresent) {
                ResponseEntity<Project>(projectData.get(), HttpStatus.OK)
            } else {
                ResponseEntity<Project>(HttpStatus.NOT_FOUND)
            }
        } else {
            ResponseEntity<Project>(HttpStatus.FORBIDDEN)
        }
    }

    @PostMapping("/projects")
    fun addProject(@RequestBody project: Project): ResponseEntity<Project> {
        return try {
            val found = projectRepository.countByName(project.name)
            if (project.name.isNotBlank()) {
                if (project.name.length > 100 || project.description!!.length > 255) {//too long
                    ResponseEntity<Project>(HttpStatus.PAYLOAD_TOO_LARGE)
                } else if (found > 0) {//duplicate
                    ResponseEntity<Project>(HttpStatus.CONFLICT)
                } else {
                    val username: String = SecurityContextHolder.getContext().authentication.name
                    val userLogin = userRepository.getUserByUsername(username)
                    project.name = project.name.trim()
                    project.owner = userLogin
                    val saveProject: Project = projectRepository.save(project)
                    ResponseEntity<Project>(saveProject, HttpStatus.CREATED)
                }
            } else {//name invalid
                ResponseEntity<Project>(HttpStatus.BAD_REQUEST)
            }
        } catch (e: Exception) {
            ResponseEntity<Project>(null, HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

    @PutMapping("/projects/{id}")
    fun updateProjectById(@PathVariable("id") id: Long, @RequestBody project: Project): ResponseEntity<out Any?> {
        val projectData = projectRepository.findById(id)
        val found = projectRepository.countByName(project.name)
        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
        val validOwner = projectRepository.validOwner(id.toString(), userId.id.toString())
        val validAdmin = memberRepository.validRole(id.toString(), userId.id.toString(), "2")
        return if (projectData.isPresent) {
            if (validOwner > 0 || validAdmin > 0) {
                if (project.name.isNotBlank()) {
                    if (project.name.length > 100 || project.description!!.length > 255) {//too long
                        ResponseEntity<Project>(HttpStatus.PAYLOAD_TOO_LARGE)
                    } else {
                        if ((project.name == projectData.get().name && found > 0) || (project.name !== projectData.get().name && found == 0)) {
                            val saveProject = projectData.get()
                            saveProject.name = project.name.trim()
                            saveProject.description = project.description
                            ResponseEntity<Any?>(projectRepository.save(saveProject), HttpStatus.OK)
                        } else {//duplicate
                            ResponseEntity<Project>(HttpStatus.CONFLICT)
                        }
                    }
                } else {//name invalid
                    ResponseEntity<Project?>(HttpStatus.BAD_REQUEST)
                }
            } else {
                ResponseEntity<Project>(HttpStatus.FORBIDDEN)
            }
        } else {
            ResponseEntity<Project?>(HttpStatus.NOT_FOUND)
        }
    }

    @DeleteMapping("/projects/{id}")
    fun deleteProject(@PathVariable("id") id: Long): ResponseEntity<HttpStatus> {
        val find = projectRepository.findById(id)
        val used = platformRepository.findByProjectId(id.toString()).size
        val userId = userRepository.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
        val validOwner = projectRepository.validOwner(id.toString(), userId.id.toString())
        return try {
            if (validOwner > 0) {
                if (used > 0) {//child used in transaction
                    ResponseEntity(HttpStatus.RESET_CONTENT)
                } else if (find.isPresent) {
                    projectRepository.deleteById(id)
                    ResponseEntity(HttpStatus.OK)
                } else {//project not found
                    ResponseEntity(HttpStatus.NOT_FOUND)
                }
            } else {
                ResponseEntity(HttpStatus.FORBIDDEN)
            }
        } catch (e: Exception) {
            ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR)
        }
    }

}